# -*- coding: utf-8 -*-
from setuphelpers import *
import json
import plistlib


def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    api_url = "https://api.github.com/repos/nextcloud/desktop/releases/latest"
    download_dict = {
        "windows": "https://download.nextcloud.com/desktop/releases/Windows/Nextcloud-version-x64.msi",
        "linux": "https://download.nextcloud.com/desktop/releases/Linux/Nextcloud-version-x86_64.AppImage",
        "darwin": "https://download.nextcloud.com/desktop/releases/Mac/Installer/Nextcloud-version.pkg",
    }

    # Getting latest version information from official sources
    print("API used is: %s" % api_url)
    json_load = json.loads(wgets(api_url, proxies=proxies))
    version = json_load["tag_name"].split("-")[-1].replace("v", "")
    download_url = download_dict[control.target_os].replace("version", version)
    latest_bin = download_url.split("/")[-1]

    # Downloading latest binaries
    print("Latest %s version is: %s" % (control.name, version))
    print("Download URL is: %s" % download_url)
    if not isfile(latest_bin):
        print("Downloading: %s" % latest_bin)
        wget(download_url, latest_bin, proxies=proxies)
    else:
        print("Binary is present: %s" % latest_bin)
    
    # Checking version from file
    print("Checking if version from file is greater than version online")
    version_from_file = get_pkg_properties(latest_bin, control.name)["CFBundleVersion"]
    if Version(version_from_file, 4) > Version(version, 4):
        print("Changed version to version from file")
        version = version_from_file
    else:
        print("Version from file is equal to version online")

    # Changing version of the package
    if Version(version, 4) > Version(control.get_software_version(), 4):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        package_updated = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Deleting outdated binaries
    remove_outdated_binaries(".".join(version.split(".")[0:-1]))

    # Validating update-package-sources
    return package_updated


def get_pkg_properties(pkg_path, app_name):
    '''
    Get the properties of a .pkg file by getting the Info.plist of the package

    Args:
        pkg_path: path to .pkg file
        app_name: name of the app contained in the .pkg file

    Returns:
        plist_read: dict of Info.plist file
    '''
    
    # Extracting Info.plist file from .pkg file
    _7zip = "/usr/local/bin/7z"
    plist_path = makepath(f"*{app_name}.app", "Contents", "Info.plist")
    run(rf"{_7zip} e {pkg_path} Payload* -y")
    run(rf"{_7zip} e .{os.sep}Payload* {plist_path} -r -y")

    plist_read = plistlib.readPlist("Info.plist")

    remove_file("*Payload*")
    remove_file("Info.plist")

    return plist_read
